@@ -6,6 +6,9 @@ from labelbox.schema.data_row_metadata import DataRowMetadataKind
 from google.api_core import retry
 import math
 import uuid
+from datetime import datetime
+from dateutil import parser
+import pytz
 
 
 class Client:
@@ -41,11 +44,12 @@ class Client:
         # Check to make sure the value in your metadata index is one of the accepted values        
         connector.enforce_metadata_index(metadata_index, verbose)
         # If your table doesn't have columns for all your metadata_field_names, make columns for them
-        if metadata_index:
-            column_names = get_columns_function(table)
-            for metadata_field_name in metadata_index.keys():
-                if metadata_field_name not in column_names:
-                    table = add_column_function(table, column_name=metadata_field_name, default_value=None)
+        if type(table) != bool:
+            if metadata_index:
+                column_names = get_columns_function(table)
+                for metadata_field_name in metadata_index.keys():
+                    if metadata_field_name not in column_names:
+                        table = add_column_function(table, column_name=metadata_field_name, default_value=None)
         # If Labelbox doesn't have metadata for all your metadata_field_names, make Labelbox metadata fields
         for metadata_field_name in metadata_index.keys():
             metadata_type = metadata_index[metadata_field_name]
@@ -178,55 +182,79 @@ class Client:
             if verbose:
                 print(f'Created a new dataset with ID {dataset.uid}') 
         return dataset      
-
-    def get_ontology_schema_to_name_path(self, ontology_normalized:dict, divider:str="///", invert:bool=False):
+    
+    def get_ontology_schema_to_name_path(ontology, divider:str="///", invert:bool=False, detailed:bool=False):
         """ Recursively iterates through an ontology to create a dictionary where {key=schema_id : value=name_path}
         Where name_path = parent{divider}answer{divider}parent{divider}answer.... where divider="///"
         Args:
-            ontology_normalized     :   Required (dict) : Ontology as a dictionary from ontology.normalized (where type(ontology) == labelbox.schema.ontology.Ontology)
+            ontology_normalized     :   Required (dict or labelbox.schema.ontology.Ontology) - Either Labelbox Ontology object or labelbox.schema.ontology.Ontology.normalized dictionary
             divider                 :   Optional (str) - String delineating the tool/classification/answer path for a given schema ID
-            invert                  :   Optional (bool) : If True, will invert the dictionary to where {key=name_path : value=schema_id}
+            invert                  :   Optional (bool) - If True, will invert the dictionary to where {key=name_path : value=schema_id}
+            detailed                :   Optional (bool) - If True, the dictionary will have the same key, but the values will be {"name" "color" "type" "kind" "parent_schema_ids"}
         Returns:
-            Dictionary where {key=schema_id : value=name_path} - or the inverse
+            Dictionary where {key=schema_id : value=name_path} - or the inverse - value can be more detailed if detailed=True
         """
-        def map_layer(feature_dict:dict={}, node_layer:list= [], parent_name_path:str="", divider:str="///"):
+        def map_layer(feature_dict:dict={}, node_layer:list= [], parent_name_path:str="", divider:str="///", invert:bool=False, detailed:bool=False, encoded_value:int=0):
             """ Recursive function that does the following for each node in a node_layer:
                     1. Creates a name_path given the parent_name_path
                     2. Adds schema_id : name_path to your working dictionary
                     3. If there's another layer for a given node, recursively calls itself, passing it its own name key as it's childrens' parent_name_path
             Args:
-                feature_dict              :     Dictionary where {key=schema_id : value=name_path}
-                node_layer                :     A list of classifications, tools, or option dictionaries
-                parent_name_path           :     A concatenated list of parent node names separated with "///" creating a unique mapping key
+                feature_dict        :   Required (dict) - Building dictionary of ontology information
+                node_layer          :   Required (list) - A list of ontology classification, tool, or option dictionaries
+                parent_name_path    :   Required (str) - A concatenated list of parent node names separated with `divider` value, creating a unique mapping key
+                invert              :   Optional (bool) - If True, will invert the dictionary to where {key=name_path : value=schema_id}
+                divider             :   Optional (str) - String delineating the tool/classification/answer path for a given schema ID
+                detailed            :   Optional (bool) - If True, the dictionary will have the same key, but the values will be {"name" "color" "type" "kind" "parent_schema_ids"}
+                encoded_value       :   Optional (int) - Counts the number of nodes in an ontology, creating an encoded numerical value for schemas
             Returns:
                 feature_dict
             """
             if node_layer:
                 for node in node_layer:
+                    encoded_value += 1
                     if "tool" in node.keys():
                         node_name = node["name"]
                         next_layer = node["classifications"]
-                    elif "instructions" in node.keys():
-                        node_name = node["instructions"]
-                        next_layer = node["options"]
+                        node_type = node["tool"]
+                    node_kind = "tool"   
+                elif "instructions" in node.keys():
+                    node_name = node["instructions"]
+                    next_layer = node["options"]
+                    node_kind = "classification"
+                    node_type = node["type"]                        
+                else:
+                    node_type = "option"
+                    node_name = node["label"]
+                    next_layer = node.get("options", [])
+                    node_kind = "branch_option" if next_layer else "leaf_option" 
+                name_path = f"{parent_name_path}{divider}{node_name}" if parent_name_path else node_name
+                dict_key = node['featureSchemaId'] if invert else name_path
+                dict_value = name_path if invert else node['featureSchemaId']
+                if detailed:
+                    dict_value = {"type":node_type,"kind":node_kind, "encoded_value" : encoded_value}
+                    if invert:
+                        dict_value["name_path"]=name_path
                     else:
-                        node_name = node["label"]
-                        next_layer = node["options"] if 'options' in node.keys() else []
-                    if parent_name_path:
-                        name_path = parent_name_path + divider + node_name
-                    else:
-                        name_path = node_name
-                    feature_dict.update({node['featureSchemaId'] : name_path})
-                    if next_layer:
-                        feature_dict = map_layer(feature_dict, next_layer, name_path. divider)
-            return feature_dict
-        ontology_schema_to_name_path = map_layer(feature_dict={}, node_layer=ontology_normalized["tools"], divider=divider) if ontology_normalized["tools"] else {}
-        if ontology_normalized["classifications"]:
-            ontology_schema_to_name_path = map_layer(feature_dict=ontology_schema_to_name_path, node_layer=ontology_normalized["classifications"], divider=divider)
-        if invert:
-            return {v: k for k, v in ontology_schema_to_name_path.items()} 
-        else:
-            return ontology_schema_to_name_path        
+                        dict_value["schema_id"]=node['featureSchemaId']
+                feature_dict.update({dict_key : dict_value})
+                if next_layer:
+                    feature_dict, encoded_value = map_layer(feature_dict, next_layer, name_path, divider, invert=invert, detailed=detailed, encoded_value=encoded_value)
+        return feature_dict, encoded_value
+    if type(ontology) == labelbox.Ontology:
+        ontology_normalized = ontology.normalized
+    elif type(ontology) == dict:
+        ontology_normalized = ontology
+    else:
+        raise TypeError(f"Input for ontology must be either a Lablbox ontology object or a dictionary representation of a Labelbox ontology - received input of type {ontology}") 
+    if ontology_normalized["tools"]:
+        working_dictionary, working_encoded_value = map_layer(feature_dict={}, node_layer=ontology_normalized["tools"], divider=divider, invert=invert, detailed=detailed)
+    else:
+        working_dictionary = {} 
+        working_encoded_value = 0
+    if ontology_normalized["classifications"]:
+        working_dictionary, working_encoded_value = map_layer(feature_dict=working_dictionary, node_layer=ontology_normalized["classifications"], divider=divider, invert=invert, detailed=detailed, encoded_value=working_encoded_value)
+    return working_dictionary
  
     def batch_upload_annotations(self, project:labelboxProject, annotations:list, import_name:str=str(uuid.uuid4()), how:str="MAL", batch_size:int=20000, verbose=False):
         """ Batch imports labels given a batch size via MAL or LabelImport
@@ -282,7 +310,6 @@ class Client:
                     print(f'Success: upload batch number {batch_number} complete')               
         return []
 
-    ## Update for datetime later
     def process_metadata_value(self, metadata_value, metadata_type:str, parent_name:str, metadata_name_key_to_schema:dict, divider:str="///"):
         """ Processes inbound values to ensure only valid values are added as metadata to Labelbox given the metadata type. Returns None if invalid or None
         Args:
@@ -305,14 +332,19 @@ class Client:
                 return_value = metadata_name_key_to_schema[name_key]
             else:
                 return_value = None                  
-        elif metadata_type == "number": # For numbers, its ints as strings
+        elif metadata_type == "number": # For numbers, it's ints as strings
             try:
                 return_value = str(int(row_value))
             except:
                 return_value = None                  
         elif metadata_type == "string": 
             return_value = str(metadata_value)
-        else: ## Update for datetime later
-            return_value = metadata_value      
+        else: # For datetime, it's an isoformat string
+            if type(metadata_value) == str:
+                return_value = parser.parse(metadata_value).astimezone(pytz.utc).replace(tzinfo=None)
+            elif type(metadata_value) == datetime.datetime:
+                return_value = metadata_value.astimezone(pytz.utc).replace(tzinfo=None)
+            else:
+                return_value = None       
         return return_value    
     
